
// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "HUB Server"-Application (for Raspberry PI with Java SE Runtime)
//
//
package com.teamo.biom.hub;
// ----- Imported Libraries -----
    import java.util.ArrayList;

public class ModuleList
{

    static final int[][] MODULE_TYPE_PRESET = {
    //    LED ON    LED OFF     Humidity 
        { 3*60,     21*60,      450 },      // Type 0: Garden Cress
        { 1*60,     8*60,       400 },      // Type 1: Champignons
        { 10*60,    14*60,      450 },      // Type 2: Strawberries
        { 14*60,    10*60,      450 },      // Type 3: Lettuce
        { 14*60,    10*60,      450 }       // Type 4: Radishes
    };
    static final int MODULE_TYPE_COUNT = MODULE_TYPE_PRESET.length;

    // SENSOR IDs
    static final int SENSOR_COUNT = 2;
    static final int SENSOR_ID_HUMIDITY = 0;
    static final int SENSOR_ID_WATER_TANK = 1;

    static ModuleList ml = new ModuleList();
    static ArrayList<Module> modules = new ArrayList<Module>();

    static void makeTestBoxes()
    {
        modules.add( ml.new Module( 1001, 3, 1 ) );
        modules.add( ml.new Module( 1002, 2, 1 ) );
    }

    private class Module
    {
        // General Fields
            long id;
            int type;
            int[] sensorData;
            int ledPowerOn, ledCycleState;

            long lastCall;

        public Module( long initID, int initType, long initCallId )
        {
            this.id = initID;
            this.type = initType;
            this.sensorData = new int[ SENSOR_COUNT ];
            this.lastCall = initCallId;
        }

        public void renewLastCall( long callId )
        {
            this.lastCall = callId;
        }
    }

    static void updateActuatorState( long id, int actId, int actState )
    {
        for ( Module m : modules )
        {
            if ( m.id == id )
            {
                int index = (int)m.id - 1001;
                if ( actId == 0 )
                {
                    m.ledPowerOn = actState;
                    
                    SerialCommunicator.transmit(SerialCommunicator.ports[index], "2");
                    //SerialCommunicator.transmit(SerialCommunicator.ports[m.id], "2");
                } else {
                    SerialCommunicator.transmit(SerialCommunicator.ports[index], "1");
                    //SerialCommunicator.transmit(SerialCommunicator.ports[m.id], "1");
                }
                    // if ( !port.isOpen() )
                    // {
                    //     port.setBaudRate(9600);
                    //     port.openPort();
                    // }
                    // try (
                    //     OutputStream out = port.getOutputStream();
                    // ) {
                    //     out.write( 'l' );
                    // } catch ( Exception e ) {
                    //     System.out.println( "ModuleList to SerialCommunicator: An error occured while attempting to turn the LED of Module " + m.id + "on." );
                    //     System.out.println( e.getMessage() );
                    // }

                    //SerialCommunicator.transmit( m.port, HubServer.constructProtocol( "MAP", content) );
                
            }
        }
    }

    static void registerModule( long initId, int initType, long callId )
    {
        boolean isNewModule = true;
        for ( Module m : modules )
        {
            if ( m.id == initId )
            {
                isNewModule = false;
                m.renewLastCall( callId );
            }
        }

        if ( isNewModule )
        {
            modules.add( ml.new Module( initId, initType, callId ) );
            System.out.println( "ModuleList: The Module " + initId + " has been registered." );
        }
    }

    static void feedSensorData( long id, int sensorId, int sensorInput )
    {
        for ( Module m : modules )
        {
            if ( m.id == id )
            {
                if ( sensorId < SENSOR_COUNT )
                    m.sensorData[ sensorId ] = sensorInput;
            }
        }
    }

    // Class for cleaning disconnected modules from the module list.
    // Runs based on a process timer in controller.
    public class Cleaner extends Thread
    {
        public void run()
        {
            for ( Module m : modules )
            {
                if ( m.lastCall < SerialCommunicator.lastCallId-2 )
                    modules.remove( m );
            }
        }
    }

    static String[] getModulePackage()
    {
        ArrayList<String> tempCont = new ArrayList<String>();

        for ( Module m : modules )
        {
            tempCont.add( Long.toString(     m.id ) );
            tempCont.add( Integer.toString(  m.type ) );
            tempCont.add( Integer.toString(  m.ledPowerOn ) );
            tempCont.add( Integer.toString(  m.ledCycleState ) );
            tempCont.add( Integer.toString(  m.sensorData[ SENSOR_ID_HUMIDITY ] ) );
            tempCont.add( Integer.toString(  m.sensorData[ SENSOR_ID_WATER_TANK ] ) );
        }

        String[] content = new String[ tempCont.size() ];
        tempCont.toArray( content );

        return content;
    }

}