

class GraphicUserInterface
{
    
    PFont mainFont;
    GScreen[] screens;
    
    GraphicUserInterface()
    {
        mainFont = createFont( "Neutronium.ttf", 32 );
    }


    void loadScreens()
    {
        screens = new GScreen[2];
        screens[SEARCHSCREEN] = new SearchScreen();
        screens[MAINSCREEN] = new MainScreen();
    }
    
    void update()
    {
        if ( Controller.isTCPOnline() )
        {
            screens[MAINSCREEN].show();
            screens[SEARCHSCREEN].hide();
        }
        else {
            screens[SEARCHSCREEN].show();
            screens[MAINSCREEN].hide();
        }
    }

    void render()
    {
        for ( GScreen s : screens )
        {
            if ( s.isActive() ) s.render();
        }
    }

    class GScreen
    {
        String name;
        boolean active;

        GScreen( String initName )
        {
            this.name = initName;
        }

        boolean isActive()
        {
            return active;
        }

        void show()
        {
            active = true;
        }
        void hide()
        {
            active = false;
        }

        void render(){}
    }

    class GText
    {
        String txt;
        float fontSize;
        PVector position;
        
        GText( String initString, float initX, float initY, float initSize )
        {
            this.txt = initString;
            this.position = new PVector( initX, initY );
            this.fontSize = initSize;
        }

        void write()
        {
            float x = this.position.x;
            float y = this.position.y;
            textFont( mainFont, this.fontSize );
            text( this.txt, x, y );
        }
    }

    class GPopup
    {
        String[] contents;
        boolean active;
        PVector position;
        PVector size;

        GPopup( String initFile )
        {
            this.contents = loadStrings( initFile );
            position = new PVector( width/2, height/2 );
            size = new PVector( width*0.8, height*0.8 );
        }

        void render()
        {
            float x = this.position.x;
            float y = this.position.y;
            float w = this.size.x;
            float h = this.size.y;

            fill            (255);
            stroke          (0);
            strokeWeight    (2);
            rect            ( x, y, w, h );

            strokeWeight    (1);
        }

        void show()
        {
            this.active = true;
        }

        void hide()
        {
            this.active = false;
        }

        boolean isOpen()
        {
            return active;
        }
    }

    class GiButton
    {
        PShape icon;
        PVector position;

        GiButton( float initX, float initY )
        {
            this.icon = loadShape( "iButton.svg" );
            this.position = new PVector( initX, initY );
        }

        void render()
        {
            float x = this.position.x;
            float y = this.position.y;

            shape( this.icon, x, y );
        }

        boolean hovered( float ux, float uy )
        {
            float bx = this.position.x;
            float by = this.position.y;
            float bw = this.icon.width;
            float bh = this.icon.height;

            if ( ux >= bx - (bw/2) && ux <= bx + (bw/2) )
            {
                if ( uy >= by - (bh/2) && uy <= by + (bh/2) )
                {
                    return true;
                }
            }
            return false;
        }
    }

    class GLogo
    {
        PImage logo;
        PVector position;

        GLogo( float initX, float initY )
        {
            this.logo = loadImage( "logo.png" );
            this.position = new PVector( initX, initY );
        }

        GLogo( float initX, float initY, boolean whiteMode )
        {
            this.position = new PVector( initX, initY );

            String path = "logo.png";
            if ( whiteMode ) path = "logo_white.png";
            
            this.logo = loadImage( path );
        }

        void render()
        {
            float x = this.position.x;
            float y = this.position.y;

            image( this.logo, x, y );
        }

    }

}