

class MainScreen extends GraphicUserInterface.GScreen
{

    GraphicUserInterface.GLogo logo;
    PImage[] typeIcon = new PImage[ ModuleList.MODULE_TYPE_COUNT ];
    PImage waterTankIcon, humidityIcon, ledIcon;

    MainScreen()
    {
        gui.super( "MainScreen" );

        float logoX = width/2;
        float logoY = ( height* 0.1 );
        logo = gui.new GLogo( logoX, logoY, true );

        for ( int i= 0; i < ModuleList.MODULE_TYPE_COUNT; i++ )
        {
            typeIcon[i] = loadImage( "type" + i + ".png" );
        }

        waterTankIcon = loadImage( "waterTankIcon.png" );
        humidityIcon = loadImage( "humidityIcon.png" );
        ledIcon = loadImage( "ledIcon.png" );
    }

    void render()
    {
        background( 87 );
        this.update();
        this.display();
    }

    void update(){}

    void display()
    {
        drawModuleBoxes();

        logo.render();
    }

    void drawModuleBoxes()
    {
        int i = 0;
        ModuleList.Module[] mods = new ModuleList.Module[ModuleList.modules.size()];
        ModuleList.modules.toArray( mods );

        for ( ModuleList.Module m : mods )
        {
            float w = width*0.4;
            float h = w;
            float x = width*0.05 + w/2;
            float y = height*0.25 + i*(h+10);
            float r = 10;

            pushMatrix();
                translate( x, y );
                noStroke();

                // Water Pipe
                if ( i != ModuleList.modules.size()-1 )
                {
                    fill( 150, 191, 196 );
                        float wpW = w/8;
                        float wpH = 20;
                        float wpX = 0 - 2.5*wpW;
                        float wpY = 0 + 4.5*wpW;
                        rect( wpX, wpY, wpW, wpH );
                }

                // Box Body
                    fill( 234 );
                        rect( 0, 0, w, h, r );

                    fill( 255 );
                        rect( 0, 0, w-w/5, h-h/5, r );

                    fill( 234 );
                        rect( 0, 0-(h/2)+(h/7), w*2/4, h-(8*h/9), r/2 );

                // Type Icon
                    if ( typeIcon[i] != null )
                    {
                        float iconY = 0 + h/2 - h/10 - typeIcon[i].height/2;
                        image( typeIcon[m.type], 0, iconY );                      
                    }

            popMatrix();

            pushMatrix();
                // Info Body
                    float infoW = w;
                    float infoH = 0.6*h;
                    float infoX = width - x;
                    float infoY = y;

                    translate( infoX, infoY );

                    // Shadow
                        noStroke();
                        fill( 80, 100 );
                            rect( 0, 0+5, infoW, infoH, r*2 );
                        fill( 70, 100 );
                            rect( 0, 0+4, infoW, infoH, r*2 );
                        fill( 50, 100 );
                            rect( 0, 0+2, infoW, infoH, r*2 );

                    // Box
                        fill( 255 );
                        noStroke();
                            rect( 0, 0, infoW, infoH, r*2 );

                    int[] data = m.getData();

                    float iX1 = 0 - infoW/2 + infoW/8;
                    float iX2 = 0 + infoW/2 - (3*infoW)/4;
                    float iY = 0 - infoH/3;
                    float barLength = 0.8* (3*infoW/4);
                    // Water Tank Data
                        image( waterTankIcon, iX1, iY );

                        rectMode( CORNER );
                        fill( 240 );
                            rect( iX2, iY+2, barLength, 4, 5 );

                        float waterTankPercent = (data[1]*25)-min(12.5,data[1]*12.5);
                        float waterTankLength = (waterTankPercent/100) * barLength;
                        fill( 150, 191, 196 );
                            rect( iX2, iY+2, waterTankLength, 4, 5 );

                        textSize( 10 );
                        textAlign( LEFT, BOTTOM );
                        text( waterTankPercent+"%", iX2, iY );

                    iY += infoH/3;
                    // Humidity Data
                        image( humidityIcon, iX1, iY );

                        fill( 240 );
                            rect( iX2, iY, barLength, 4, 5 );
                        
                        float flooredData = max(0, data[0] - 300), dataMax = 400;
                        float flooredThreshold = ModuleList.MODULE_TYPE_PRESET[m.getType()][2] - 300;

                        float humidityThresholdX = (flooredThreshold/dataMax) * barLength;
                        stroke( 100 );
                        strokeWeight( 2 );
                            line( iX2+humidityThresholdX, iY, iX2+humidityThresholdX, iY+4 );
                        
                        float humidityX = (flooredData/dataMax)*barLength;
                        stroke( 83, 144, 31 );
                        noFill();
                            ellipse( iX2+humidityX, iY+2, 8, 8 );
                        
                        strokeWeight( 1 );

                    iY += infoH/3;
                    // LED Data
                        image( ledIcon, iX1, iY );

                        String txt = "";
                        color col = color(0);
                        textSize( 8 );
                        if ( data[2] == 0 )
                        {
                                txt = "Power Off";
                                col = color(111);
                        } else {
                                txt = "Power On";
                                col = color(83, 144, 31);
                        }
                        
                        fill( col );
                        textAlign( LEFT, CENTER );
                        textSize( 10 );
                        text( txt, iX2-6, iY-2 );

                        if ( data[3] == 0 )
                        {
                                txt = "Cycle Off";
                                col = color(111);
                        } else {
                                txt = "Cycle On";
                                col = color(83, 144, 31);
                        }
                        
                        fill( col );
                        textAlign( RIGHT, CENTER );
                        text( txt, iX2+barLength, iY-2 );

                    rectMode( CENTER );
                    
            popMatrix();

            i++;
        }
    }
}