

class ModuleChanger
{
    static final int INPUT_TYPE_LED_POWER = 0;
    static final int INPUT_TYPE_WATER_TANK = 1;

    PVector center, corner;
    boolean active;
    ModuleList.Module selectedModule;

    LedButton ledButton;
    WateringButton wateringButton;

    ModuleChanger()
    {
        this.center = new PVector( width/2, height/2 );
        this.corner = new PVector( width/2-getWidth()/2, height/2-getHeight()/2 );
        this.selectedModule = null;
        this.ledButton = new LedButton( "LED Power", this.corner.x+getWidth()/4, this.corner.y+getHeight()/10, 60 );
        this.wateringButton = new WateringButton( "Water Plant", this.center.x+getWidth()/4, this.corner.y+getHeight()/10, 60 );
    }

    void changeLedPowerState()
    {
        String[] content = {
            "" + this.selectedModule.id,
            "" + INPUT_TYPE_LED_POWER,
            "" + ledButton.state
        };

        TCPClient.transmit( TCPClient.constructProtocol( "UIP", content ));
    }

    void waterModule()
    {
        String[] content = {
            "" + this.selectedModule.id,
            "" + INPUT_TYPE_WATER_TANK,
            "" + 1
        };

        TCPClient.transmit( TCPClient.constructProtocol( "UIP", content ));
    }

    void display()
    {
        
        float w = this.getWidth();
        float h = this.getHeight();

        rectMode( CENTER );
        fill( 255, 200 );
        stroke( 150 );
        strokeWeight( 2 );
            rect( this.center.x, this.center.y, w, h, 10 );

        ledButton.render();
        wateringButton.render();

    }

    void hide()
    {
        this.active = false;
    }
    void show( ModuleList.Module m )
    {
        this.active = true;
        this.selectedModule = m;
        this.ledButton.setState( m.ledPowerOn );
    }
    boolean isActive()
    {
        return this.active;
    }

    ModuleList.Module boxClicked( float mx, float my )
    {
        float w = width*0.4;
        float h = w;
        float x = width*0.05 + w/2;
        int i = 0;
        for ( ModuleList.Module m : ModuleList.modules )
        {
            float y = height*0.25 + i*(h+10);

            if ( mx >= x-w/2 && mx <= x+w/2 ){
                if ( my >= y-h/2 && my <= y+h/2 ) {
                    return m;
                }
            }
            i++;
        }
        return null;
    }

    boolean clickedOutside( float mx, float my )
    {
        float w = this.getWidth();
        float h = this.getHeight();
        float x = width/2;
        float y = height/2;

        if (    mx < this.corner.x
             || mx > this.corner.x+w 
             || my < this.corner.y
             || my > this.corner.y+h )
        { 
            return true;
        }
        return false;
    }

    float getWidth()
    {
        return 0.7*width;
    }
    float getHeight()
    {
        return 0.7*height;
    }

    abstract class Button
    {
        boolean pressed;
        PVector position;
        float radius;
        int state;
        String label;
        int[][] innerCol = new int[2][3];

        void setState( int initState )
        {
            this.state = initState;
        }

        void render()
        {
            float x = this.position.x;
            float y = this.position.y;
            float rad = this.radius;
            
            color col = color(255);

            int r = innerCol[ state ][0], g = innerCol[ state ][1], b = innerCol[ state ][2];
            if ( pressed ) 
            {
                r+= 50;
                g*= 50;
                b+= 50;
            }
            
            fill( r, g, b );
                ellipse( x, y, rad, rad );
            noFill();
            stroke( 0 );
                ellipse( x, y, rad-20, rad-20 );
                //arc( x, y, r-10, r-10, 3*PI/4, PI/4);

            fill( 0 );
            textAlign( CENTER, CENTER );
            textSize( 18 );
                text( label, x, y +3*rad/4 );
            
        }

        boolean buttonClicked( float mx, float my )
        {
            float x = this.position.x;
            float y = this.position.y;
            float r = this.radius;

            if ( dist( x, y, mx, my) < r/2 )
                return true;

            return false;
        }
        void updateButtonState()
        {
            this.state++;
            if ( this.state > 1 ) this.state = 0;
        }
    }

    private class LedButton extends Button
    {       
        LedButton( String initLabel, float initX, float initY, float initR )
        {
            this.position = new PVector( initX, initY );
            this.radius = initR;
            this.label = initLabel;

            this.innerCol[1][0] = 100;
            this.innerCol[1][1] = 100;
            this.innerCol[1][2] = 100;

            this.innerCol[0][0] = 83;
            this.innerCol[0][1] = 144;
            this.innerCol[0][2] = 31;
        }
    }

    private class WateringButton extends Button
    {
        WateringButton( String initLabel, float initX, float initY, float initR )
        {
            this.position = new PVector( initX, initY );
            this.radius = initR;
            this.label = initLabel;

            this.innerCol[0][0] = 100;
            this.innerCol[0][1] = 100;
            this.innerCol[0][2] = 100;

            this.innerCol[1][0] = 200;
            this.innerCol[1][1] = 200;
            this.innerCol[1][2] = 200;
        }
    }
}