

class SearchScreen extends GraphicUserInterface.GScreen
{
    // Unique Screen Objects
    Phone phone;

    ArrayList<TriWave> triWaves = new ArrayList<TriWave>();
    int triWaveTimer, triWaveTimerCheck, triWaveDelay;

    // Common Screen Objects
    GraphicUserInterface.GLogo logo;
    GraphicUserInterface.GText textScanning;
    GraphicUserInterface.GiButton iButton;
    GraphicUserInterface.GPopup helpPopup;

    SearchScreen()
    {
        gui.super( "SearchScreen" );
        this.show();

        this.phone = new Phone( width/2, height/2 );

        this.triWaveDelay = 3000;

        float logoX = width/2;
        float logoY = ( height* 0.1 );
        logo = gui.new GLogo( logoX, logoY );

        float scanningX = width/2;
        float scanningY = height - (0.2*height);
        textScanning = gui.new GText( "SCANNING...", scanningX, scanningY, 32 );

        float iButtonX = width - ( 0.1*width );
        float iButtonY = ( 0.05*height );
        iButton = gui.new GiButton( iButtonX, iButtonY );

        helpPopup = gui.new GPopup( "searchscreenhelp.txt" );
    }

    void render()
    {
        background( 255 );
        this.update();
        this.display();
    }

    void update()
    {
        triWaveTimer = int(ms);
        if ( (triWaveTimer - triWaveTimerCheck) >= triWaveDelay )
        {
            triWaveTimerCheck = triWaveTimer;
            triWaves.add( new TriWave() );
        }

        for ( int i = 0; i < triWaves.size(); i++ )
        {
            TriWave tw = triWaves.get(i);
            tw.update();
            if ( tw.outOfScreen() )
            {
                triWaves.remove( i );
                i--;
            }
        }
    }

    void display()
    {
        for ( TriWave tw : triWaves )
        {
            tw.render();
        }

        phone.render();

        fill( 0 );
        textScanning.write();

        iButton.render();

        logo.render();
    }

    class Phone
    {
        PImage icon;
        PVector position;

        Phone( float initX, float initY )
        {
            this.icon = loadImage( "phone.png" );
            this.position = new PVector( initX, initY );
        }

        void render() 
        {
            float x = this.position.x;
            float y = this.position.y;

            image( this.icon, x, y );
        }
    }

    class TriWave
    {

        float radius[];

        TriWave()
        {
            radius = new float[3];
            for ( int i = 0; i < 3; i++ )
            {
                radius[i] = 1 - i*40;
            }
        }

        void update()
        {
            for ( int i = 0; i < 3; i++ )
            {
                radius[i] += 4;
            }
        }

        void render()
        {
            for ( int i = 0; i < 3; i++ )
            {
                if ( radius[i] > 0 )
                {
                    pushMatrix();
                        noFill();
                        strokeWeight( 2 );

                            float multiplier = radius[i]/500;
                            float green = multiplier * 255;
                            color col = color( 0, green, 0);
                        stroke( col );

                        translate( width/2, height/2 );

                        ellipse( 0, 0, radius[i], radius[i] );

                        strokeWeight( 1 );
                    popMatrix();
                }
            }
        }

        boolean outOfScreen()
        {
            if ( this.radius[2] > height+100 ) return true;
            return false;
        }

    }

}