import processing.core.*; 
import processing.data.*; 
import processing.event.*; 
import processing.opengl.*; 

import java.util.HashMap; 
import java.util.ArrayList; 
import java.io.File; 
import java.io.BufferedReader; 
import java.io.PrintWriter; 
import java.io.InputStream; 
import java.io.OutputStream; 
import java.io.IOException; 

public class AndroidApp extends PApplet {

// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "tbd"-Application (for Android)
//
// Functions:   Finding the HUB using UDP beacons and connecting to it via TCP.
//              Receiving and displaying information about the greenhouse modules from the HUB.
//              Sending User Input to the HUB.
//
// Program Main Class "AndroidApp"
// Runs User Interface and process Threads.++
//  - Controller
//      - ServerFinder
//      - TCPClient

// Global Variables
    static long ms;

    final int SEARCHSCREEN = 0;
    final int MAINSCREEN = 1;

// Global Objects
Controller controller;
GraphicUserInterface gui;
ModuleChanger moduleChanger;

// ----- Program Setup -----
public void setup()
{
    
    //fullScreen();
        ellipseMode ( CENTER );
        imageMode   ( CENTER );
        rectMode    ( CENTER );
        textAlign   ( CENTER, CENTER );
        shapeMode   ( CENTER );

    gui = new GraphicUserInterface();
    gui.loadScreens();
    moduleChanger = new ModuleChanger();

    // Setup File Reader
        new FileReader( this );

    // Attempt Initial TCP Connection
        //Controller.connectToTCP( Controller.getHubIP() );

    // Start Controller Thread
        controller = new Controller();
        controller.start();


    println( "Main: Program Setup complete. Application running." );
}

// ----- Main Loop -----
public void draw()
{
    SYS_update();
    //Controller.updateTCP( true );

    gui.update();
    gui.render();
    if ( moduleChanger.isActive() )
        moduleChanger.display();
}

public void SYS_update()
{
    ms = millis();
}

// ----- Event Handling -----
public void mousePressed()
{
    float mx = mouseX, my = mouseY;

    // if ( gui.screens[SEARCHSCREEN].helpPopup.isOpen() ) gui.screens[SEARCHSCREEN].helpPopup.hide();
    // if ( gui.screens[SEARCHSCREEN].iButton.hovered() ) gui.screens[SEARCHSCREEN].helpPopup.show();

    if ( gui.screens[ MAINSCREEN ].isActive() )
    {
        if ( moduleChanger.isActive() )
        {
            if ( moduleChanger.clickedOutside( mx, my ) )
                moduleChanger.hide();

            if ( moduleChanger.ledButton.buttonClicked( mx, my ) )
            {
                moduleChanger.ledButton.updateButtonState();
                moduleChanger.changeLedPowerState();
            }

            if ( moduleChanger.wateringButton.buttonClicked( mx, my ) )
            {
                moduleChanger.ledButton.updateButtonState();
                moduleChanger.waterModule();
            }
        }
        else
        {
            ModuleList.Module m = null;
            if ( (m = moduleChanger.boxClicked( mx, my )) != null )
            {
                moduleChanger.show( m );
            }
        }
    }
}


static class FileReader
{

    static PApplet app;

    FileReader( PApplet papp)
    {
        app = papp;
    }

    public static InputStream loadFile( String path )
    {
        InputStream is;
        is = app.createInput( path );
        return is;
    }

}


class GraphicUserInterface
{
    
    PFont mainFont;
    GScreen[] screens;
    
    GraphicUserInterface()
    {
        mainFont = createFont( "Neutronium.ttf", 32 );
    }


    public void loadScreens()
    {
        screens = new GScreen[2];
        screens[SEARCHSCREEN] = new SearchScreen();
        screens[MAINSCREEN] = new MainScreen();
    }
    
    public void update()
    {
        if ( Controller.isTCPOnline() )
        {
            screens[MAINSCREEN].show();
            screens[SEARCHSCREEN].hide();
        }
        else {
            screens[SEARCHSCREEN].show();
            screens[MAINSCREEN].hide();
        }
    }

    public void render()
    {
        for ( GScreen s : screens )
        {
            if ( s.isActive() ) s.render();
        }
    }

    class GScreen
    {
        String name;
        boolean active;

        GScreen( String initName )
        {
            this.name = initName;
        }

        public boolean isActive()
        {
            return active;
        }

        public void show()
        {
            active = true;
        }
        public void hide()
        {
            active = false;
        }

        public void render(){}
    }

    class GText
    {
        String txt;
        float fontSize;
        PVector position;
        
        GText( String initString, float initX, float initY, float initSize )
        {
            this.txt = initString;
            this.position = new PVector( initX, initY );
            this.fontSize = initSize;
        }

        public void write()
        {
            float x = this.position.x;
            float y = this.position.y;
            textFont( mainFont, this.fontSize );
            text( this.txt, x, y );
        }
    }

    class GPopup
    {
        String[] contents;
        boolean active;
        PVector position;
        PVector size;

        GPopup( String initFile )
        {
            this.contents = loadStrings( initFile );
            position = new PVector( width/2, height/2 );
            size = new PVector( width*0.8f, height*0.8f );
        }

        public void render()
        {
            float x = this.position.x;
            float y = this.position.y;
            float w = this.size.x;
            float h = this.size.y;

            fill            (255);
            stroke          (0);
            strokeWeight    (2);
            rect            ( x, y, w, h );

            strokeWeight    (1);
        }

        public void show()
        {
            this.active = true;
        }

        public void hide()
        {
            this.active = false;
        }

        public boolean isOpen()
        {
            return active;
        }
    }

    class GiButton
    {
        PShape icon;
        PVector position;

        GiButton( float initX, float initY )
        {
            this.icon = loadShape( "iButton.svg" );
            this.position = new PVector( initX, initY );
        }

        public void render()
        {
            float x = this.position.x;
            float y = this.position.y;

            shape( this.icon, x, y );
        }

        public boolean hovered( float ux, float uy )
        {
            float bx = this.position.x;
            float by = this.position.y;
            float bw = this.icon.width;
            float bh = this.icon.height;

            if ( ux >= bx - (bw/2) && ux <= bx + (bw/2) )
            {
                if ( uy >= by - (bh/2) && uy <= by + (bh/2) )
                {
                    return true;
                }
            }
            return false;
        }
    }

    class GLogo
    {
        PImage logo;
        PVector position;

        GLogo( float initX, float initY )
        {
            this.logo = loadImage( "logo.png" );
            this.position = new PVector( initX, initY );
        }

        GLogo( float initX, float initY, boolean whiteMode )
        {
            this.position = new PVector( initX, initY );

            String path = "logo.png";
            if ( whiteMode ) path = "logo_white.png";
            
            this.logo = loadImage( path );
        }

        public void render()
        {
            float x = this.position.x;
            float y = this.position.y;

            image( this.logo, x, y );
        }

    }

}


class MainScreen extends GraphicUserInterface.GScreen
{

    GraphicUserInterface.GLogo logo;
    PImage[] typeIcon = new PImage[ ModuleList.MODULE_TYPE_COUNT ];
    PImage waterTankIcon, humidityIcon, ledIcon;

    MainScreen()
    {
        gui.super( "MainScreen" );

        float logoX = width/2;
        float logoY = ( height* 0.1f );
        logo = gui.new GLogo( logoX, logoY, true );

        for ( int i= 0; i < ModuleList.MODULE_TYPE_COUNT; i++ )
        {
            typeIcon[i] = loadImage( "type" + i + ".png" );
        }

        waterTankIcon = loadImage( "waterTankIcon.png" );
        humidityIcon = loadImage( "humidityIcon.png" );
        ledIcon = loadImage( "ledIcon.png" );
    }

    public void render()
    {
        background( 87 );
        this.update();
        this.display();
    }

    public void update(){}

    public void display()
    {
        drawModuleBoxes();

        logo.render();
    }

    public void drawModuleBoxes()
    {
        int i = 0;
        ModuleList.Module[] mods = new ModuleList.Module[ModuleList.modules.size()];
        ModuleList.modules.toArray( mods );

        for ( ModuleList.Module m : mods )
        {
            float w = width*0.4f;
            float h = w;
            float x = width*0.05f + w/2;
            float y = height*0.25f + i*(h+10);
            float r = 10;

            pushMatrix();
                translate( x, y );
                noStroke();

                // Water Pipe
                if ( i != ModuleList.modules.size()-1 )
                {
                    fill( 150, 191, 196 );
                        float wpW = w/8;
                        float wpH = 20;
                        float wpX = 0 - 2.5f*wpW;
                        float wpY = 0 + 4.5f*wpW;
                        rect( wpX, wpY, wpW, wpH );
                }

                // Box Body
                    fill( 234 );
                        rect( 0, 0, w, h, r );

                    fill( 255 );
                        rect( 0, 0, w-w/5, h-h/5, r );

                    fill( 234 );
                        rect( 0, 0-(h/2)+(h/7), w*2/4, h-(8*h/9), r/2 );

                // Type Icon
                    if ( typeIcon[i] != null )
                    {
                        float iconY = 0 + h/2 - h/10 - typeIcon[i].height/2;
                        image( typeIcon[m.type], 0, iconY );                      
                    }

            popMatrix();

            pushMatrix();
                // Info Body
                    float infoW = w;
                    float infoH = 0.6f*h;
                    float infoX = width - x;
                    float infoY = y;

                    translate( infoX, infoY );

                    // Shadow
                        noStroke();
                        fill( 80, 100 );
                            rect( 0, 0+5, infoW, infoH, r*2 );
                        fill( 70, 100 );
                            rect( 0, 0+4, infoW, infoH, r*2 );
                        fill( 50, 100 );
                            rect( 0, 0+2, infoW, infoH, r*2 );

                    // Box
                        fill( 255 );
                        noStroke();
                            rect( 0, 0, infoW, infoH, r*2 );

                    int[] data = m.getData();

                    float iX1 = 0 - infoW/2 + infoW/8;
                    float iX2 = 0 + infoW/2 - (3*infoW)/4;
                    float iY = 0 - infoH/3;
                    float barLength = 0.8f* (3*infoW/4);
                    // Water Tank Data
                        image( waterTankIcon, iX1, iY );

                        rectMode( CORNER );
                        fill( 240 );
                            rect( iX2, iY+2, barLength, 4, 5 );

                        float waterTankPercent = (data[1]*25)-min(12.5f,data[1]*12.5f);
                        float waterTankLength = (waterTankPercent/100) * barLength;
                        fill( 150, 191, 196 );
                            rect( iX2, iY+2, waterTankLength, 4, 5 );

                        textSize( 10 );
                        textAlign( LEFT, BOTTOM );
                        text( waterTankPercent+"%", iX2, iY );

                    iY += infoH/3;
                    // Humidity Data
                        image( humidityIcon, iX1, iY );

                        fill( 240 );
                            rect( iX2, iY, barLength, 4, 5 );
                        
                        float flooredData = max(0, data[0] - 300), dataMax = 400;
                        float flooredThreshold = ModuleList.MODULE_TYPE_PRESET[m.getType()][2] - 300;

                        float humidityThresholdX = (flooredThreshold/dataMax) * barLength;
                        stroke( 100 );
                        strokeWeight( 2 );
                            line( iX2+humidityThresholdX, iY, iX2+humidityThresholdX, iY+4 );
                        
                        float humidityX = (flooredData/dataMax)*barLength;
                        stroke( 83, 144, 31 );
                        noFill();
                            ellipse( iX2+humidityX, iY+2, 8, 8 );
                        
                        strokeWeight( 1 );

                    iY += infoH/3;
                    // LED Data
                        image( ledIcon, iX1, iY );

                        String txt = "";
                        int col = color(0);
                        textSize( 8 );
                        if ( data[2] == 0 )
                        {
                                txt = "Power Off";
                                col = color(111);
                        } else {
                                txt = "Power On";
                                col = color(83, 144, 31);
                        }
                        
                        fill( col );
                        textAlign( LEFT, CENTER );
                        textSize( 10 );
                        text( txt, iX2-6, iY-2 );

                        if ( data[3] == 0 )
                        {
                                txt = "Cycle Off";
                                col = color(111);
                        } else {
                                txt = "Cycle On";
                                col = color(83, 144, 31);
                        }
                        
                        fill( col );
                        textAlign( RIGHT, CENTER );
                        text( txt, iX2+barLength, iY-2 );

                    rectMode( CENTER );
                    
            popMatrix();

            i++;
        }
    }
}


class ModuleChanger
{
    static final int INPUT_TYPE_LED_POWER = 0;
    static final int INPUT_TYPE_WATER_TANK = 1;

    PVector center, corner;
    boolean active;
    ModuleList.Module selectedModule;

    LedButton ledButton;
    WateringButton wateringButton;

    ModuleChanger()
    {
        this.center = new PVector( width/2, height/2 );
        this.corner = new PVector( width/2-getWidth()/2, height/2-getHeight()/2 );
        this.selectedModule = null;
        this.ledButton = new LedButton( "LED Power", this.corner.x+getWidth()/4, this.corner.y+getHeight()/10, 60 );
        this.wateringButton = new WateringButton( "Water Plant", this.center.x+getWidth()/4, this.corner.y+getHeight()/10, 60 );
    }

    public void changeLedPowerState()
    {
        String[] content = {
            "" + this.selectedModule.id,
            "" + INPUT_TYPE_LED_POWER,
            "" + ledButton.state
        };

        TCPClient.transmit( TCPClient.constructProtocol( "UIP", content ));
    }

    public void waterModule()
    {
        String[] content = {
            "" + this.selectedModule.id,
            "" + INPUT_TYPE_WATER_TANK,
            "" + 1
        };

        TCPClient.transmit( TCPClient.constructProtocol( "UIP", content ));
    }

    public void display()
    {
        
        float w = this.getWidth();
        float h = this.getHeight();

        rectMode( CENTER );
        fill( 255, 200 );
        stroke( 150 );
        strokeWeight( 2 );
            rect( this.center.x, this.center.y, w, h, 10 );

        ledButton.render();
        wateringButton.render();

    }

    public void hide()
    {
        this.active = false;
    }
    public void show( ModuleList.Module m )
    {
        this.active = true;
        this.selectedModule = m;
        this.ledButton.setState( m.ledPowerOn );
    }
    public boolean isActive()
    {
        return this.active;
    }

    public ModuleList.Module boxClicked( float mx, float my )
    {
        float w = width*0.4f;
        float h = w;
        float x = width*0.05f + w/2;
        int i = 0;
        for ( ModuleList.Module m : ModuleList.modules )
        {
            float y = height*0.25f + i*(h+10);

            if ( mx >= x-w/2 && mx <= x+w/2 ){
                if ( my >= y-h/2 && my <= y+h/2 ) {
                    return m;
                }
            }
            i++;
        }
        return null;
    }

    public boolean clickedOutside( float mx, float my )
    {
        float w = this.getWidth();
        float h = this.getHeight();
        float x = width/2;
        float y = height/2;

        if (    mx < this.corner.x
             || mx > this.corner.x+w 
             || my < this.corner.y
             || my > this.corner.y+h )
        { 
            return true;
        }
        return false;
    }

    public float getWidth()
    {
        return 0.7f*width;
    }
    public float getHeight()
    {
        return 0.7f*height;
    }

    abstract class Button
    {
        boolean pressed;
        PVector position;
        float radius;
        int state;
        String label;
        int[][] innerCol = new int[2][3];

        public void setState( int initState )
        {
            this.state = initState;
        }

        public void render()
        {
            float x = this.position.x;
            float y = this.position.y;
            float rad = this.radius;
            
            int col = color(255);

            int r = innerCol[ state ][0], g = innerCol[ state ][1], b = innerCol[ state ][2];
            if ( pressed ) 
            {
                r+= 50;
                g*= 50;
                b+= 50;
            }
            
            fill( r, g, b );
                ellipse( x, y, rad, rad );
            noFill();
            stroke( 0 );
                ellipse( x, y, rad-20, rad-20 );
                //arc( x, y, r-10, r-10, 3*PI/4, PI/4);

            fill( 0 );
            textAlign( CENTER, CENTER );
            textSize( 18 );
                text( label, x, y +3*rad/4 );
            
        }

        public boolean buttonClicked( float mx, float my )
        {
            float x = this.position.x;
            float y = this.position.y;
            float r = this.radius;

            if ( dist( x, y, mx, my) < r/2 )
                return true;

            return false;
        }
        public void updateButtonState()
        {
            this.state++;
            if ( this.state > 1 ) this.state = 0;
        }
    }

    private class LedButton extends Button
    {       
        LedButton( String initLabel, float initX, float initY, float initR )
        {
            this.position = new PVector( initX, initY );
            this.radius = initR;
            this.label = initLabel;

            this.innerCol[1][0] = 100;
            this.innerCol[1][1] = 100;
            this.innerCol[1][2] = 100;

            this.innerCol[0][0] = 83;
            this.innerCol[0][1] = 144;
            this.innerCol[0][2] = 31;
        }
    }

    private class WateringButton extends Button
    {
        WateringButton( String initLabel, float initX, float initY, float initR )
        {
            this.position = new PVector( initX, initY );
            this.radius = initR;
            this.label = initLabel;

            this.innerCol[0][0] = 100;
            this.innerCol[0][1] = 100;
            this.innerCol[0][2] = 100;

            this.innerCol[1][0] = 200;
            this.innerCol[1][1] = 200;
            this.innerCol[1][2] = 200;
        }
    }
}



class SearchScreen extends GraphicUserInterface.GScreen
{
    // Unique Screen Objects
    Phone phone;

    ArrayList<TriWave> triWaves = new ArrayList<TriWave>();
    int triWaveTimer, triWaveTimerCheck, triWaveDelay;

    // Common Screen Objects
    GraphicUserInterface.GLogo logo;
    GraphicUserInterface.GText textScanning;
    GraphicUserInterface.GiButton iButton;
    GraphicUserInterface.GPopup helpPopup;

    SearchScreen()
    {
        gui.super( "SearchScreen" );
        this.show();

        this.phone = new Phone( width/2, height/2 );

        this.triWaveDelay = 3000;

        float logoX = width/2;
        float logoY = ( height* 0.1f );
        logo = gui.new GLogo( logoX, logoY );

        float scanningX = width/2;
        float scanningY = height - (0.2f*height);
        textScanning = gui.new GText( "SCANNING...", scanningX, scanningY, 32 );

        float iButtonX = width - ( 0.1f*width );
        float iButtonY = ( 0.05f*height );
        iButton = gui.new GiButton( iButtonX, iButtonY );

        helpPopup = gui.new GPopup( "searchscreenhelp.txt" );
    }

    public void render()
    {
        background( 255 );
        this.update();
        this.display();
    }

    public void update()
    {
        triWaveTimer = PApplet.parseInt(ms);
        if ( (triWaveTimer - triWaveTimerCheck) >= triWaveDelay )
        {
            triWaveTimerCheck = triWaveTimer;
            triWaves.add( new TriWave() );
        }

        for ( int i = 0; i < triWaves.size(); i++ )
        {
            TriWave tw = triWaves.get(i);
            tw.update();
            if ( tw.outOfScreen() )
            {
                triWaves.remove( i );
                i--;
            }
        }
    }

    public void display()
    {
        for ( TriWave tw : triWaves )
        {
            tw.render();
        }

        phone.render();

        fill( 0 );
        textScanning.write();

        iButton.render();

        logo.render();
    }

    class Phone
    {
        PImage icon;
        PVector position;

        Phone( float initX, float initY )
        {
            this.icon = loadImage( "phone.png" );
            this.position = new PVector( initX, initY );
        }

        public void render() 
        {
            float x = this.position.x;
            float y = this.position.y;

            image( this.icon, x, y );
        }
    }

    class TriWave
    {

        float radius[];

        TriWave()
        {
            radius = new float[3];
            for ( int i = 0; i < 3; i++ )
            {
                radius[i] = 1 - i*40;
            }
        }

        public void update()
        {
            for ( int i = 0; i < 3; i++ )
            {
                radius[i] += 4;
            }
        }

        public void render()
        {
            for ( int i = 0; i < 3; i++ )
            {
                if ( radius[i] > 0 )
                {
                    pushMatrix();
                        noFill();
                        strokeWeight( 2 );

                            float multiplier = radius[i]/500;
                            float green = multiplier * 255;
                            int col = color( 0, green, 0);
                        stroke( col );

                        translate( width/2, height/2 );

                        ellipse( 0, 0, radius[i], radius[i] );

                        strokeWeight( 1 );
                    popMatrix();
                }
            }
        }

        public boolean outOfScreen()
        {
            if ( this.radius[2] > height+100 ) return true;
            return false;
        }

    }

}
  public void settings() {  size(412, 847); }
  static public void main(String[] passedArgs) {
    String[] appletArgs = new String[] { "AndroidApp" };
    if (passedArgs != null) {
      PApplet.main(concat(appletArgs, passedArgs));
    } else {
      PApplet.main(appletArgs);
    }
  }
}
