// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "tbd"-Application (for Android)
//
// "Controller"-Class (public)
// Contains methods and variables to control which threads to run.
// Passes data from and to the ConfigReader and ConfigWriter classes.

public class Controller extends Thread
{
    // ----- Control Variables -----
        static boolean udpOnline, tcpOnline;

    // ----- Thread Objects -----
        static TCPClient tcpClient;
        static ProcessTimer dataRequestTimer = new ProcessTimer(1000);

        static ServerFinder serverFinder = new ServerFinder();
        static ServerFinder.Receiver udpReceiver;
        static ServerFinder.Beacon udpBeacon;
        static ProcessTimer beaconTimer = new ProcessTimer(5000);
        
        static Thread terminator;

    // ----- Controller Main Thread -----
        @Override
        public void run()
        {
            System.out.println( "Controller: Starting up." );

            connectToTCP( getHubIP() );

            while (true)
            {
                ProcessTimer.updateMS();

                if ( isTCPOnline() != true )
                {
                    if ( beaconTimer.waitFor() )
                        {
                            udpBeacon = serverFinder.new Beacon();
                            udpBeacon.start();
                        }
                    if ( !isUDPOnline() )
                    {
                        try {
                            terminator = udpReceiver;
                            udpReceiver = null;
                            terminator.interrupt();
                        } catch ( NullPointerException e ) {}
                        try {
                            udpReceiver = serverFinder.new Receiver();
                            if ( isUDPOnline() )
                                udpReceiver.start();
                        } catch ( IllegalThreadStateException e ) {
                            System.out.println( "Controller: An error occured while attempting to start the UDP Receiver Thread." );
                            System.out.println( e.getMessage() );
                        }
                    }
                }
                else
                {
                    if ( dataRequestTimer.waitFor() )
                    {
                        String[] content = { "" };
                        TCPClient.transmit( TCPClient.constructProtocol( "DRP", content ) );
                    }

                    if ( isUDPOnline() )
                    {
                        terminator = udpReceiver;
                        udpReceiver = null;
                        terminator.interrupt();
                        updateUDP( false );
                    }
                }
            }
        }

    // ----- Read Control Methods -----
        static boolean isUDPOnline()
        {
            return udpOnline;
        }
        static boolean isTCPOnline()
        {
            return tcpOnline;
        }

    // ----- Set Control Methods -----
        static void updateUDP( boolean bool )
        {
            udpOnline = bool;
        }
        static void updateTCP( boolean bool )
        {
            tcpOnline = bool;
        }

    // ----- .conf data passing Methods
        static void rememberHUB( String initIP )
        {
            new ConfigWriter( initIP );
        }
        static String getHubIP()
        {
            return new ConfigReader().readIP();
        }

    // ----- direct connection Methods
        static void connectToTCP( String ip )
        {
            tcpClient = new TCPClient( ip );
        }
}
