// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "tbd"-Application (for Android)
//
// "HubFinder"-Class (public) (threadable process of main())
// Transmitts UDP beacons and listens for replies from the HUB.
// Passes received HUB ip to the Controller.
//  Constructor     :   opens transmitting UDP Socket
//  run method      :   Sequences threaded processes (broadcast() and listen())
//      broadcast method    :   transmitts a UDP beacon to find server
//      listen method       :   receives UDP packages from server and passes server ip to Controller
//
// ----- Imported Libraries -----

import java.net.*;
import java.io.IOException;

public class ServerFinder
{

    static final int UDP_PORT_APP_TO_SERVER = 9970;
    static final int UDP_PORT_SERVER_TO_APP = 9971;

    public class Receiver extends Thread
    {
    
        public DatagramSocket listenSocket;

        public Receiver udpReceiver;

        public Receiver()
        {
            try {
                System.out.println( "ServerFinder.Receiver: Attempting to open the UDP Socket." );

                listenSocket = new DatagramSocket( );
                listenSocket.connect( InetAddress.getLocalHost(), UDP_PORT_SERVER_TO_APP );
                
                Controller.updateUDP( true );

                System.out.println( "ServerFinder.Receiver: UDP Socket successfully opened." );
            } catch ( Exception e ) {
                System.out.println( "ServerFinder.Receiver: An error occured while attempting to open the UDP Listening Socket." );
                System.out.println( e.getMessage() );
                Controller.updateUDP( false );
            } 
        }

        @Override
        public void run()
        {
            while ( true )
            {
                try {
                    System.out.println( "ServerFinder.Receiver: Listening for Server Reply." );

                    byte[] inData = new byte[ 1024 ];                     // Initialize Data Array
                    DatagramPacket inPacket =                           // Initialize DatagramPacket for Reception
                        new DatagramPacket( inData, inData.length );

                    listenSocket.receive( inPacket) ;                     // Receive incoming Packets

                    String input = new String( inPacket.getData() );    // Convert Packet Data to String
                    if ( input != null )
                    {
                        System.out.println( "ServerFinder.Receiver: Received \"" + input + "\"." );
                        interpretData( input, inPacket.getAddress() );    // Pass Data to Interpret Method
                    }
                } catch ( IOException e ) {
                    System.out.println( e.getMessage() );
                } catch ( NullPointerException e ) {
                    System.out.println( "ServerFinder.Receiver: An error occured while listening for a server reply." );
                    System.out.println( e.getMessage() );
                }
            }
        }

        public void interpretData( String input, InetAddress ip )
        {
            if ( input.contains("ACP") )        // App Connection Protocol
            {
                System.out.println("ServerFinder: Confirmed \"" + input + "\" to be a message from the HUB.");

                String hostIP = ip.toString();              // Convert HUB IP to String

                Controller.connectToTCP( hostIP );           // Pass HUB IP to the Controller
                Controller.rememberHUB( hostIP );

                System.out.println( "ServerFinder: Successfully retrieved the HUB IP of " + hostIP );
            }
        }
    }

    public class Beacon extends Thread
    {
        @Override
        public void run()
        {
            try {
                System.out.println( "ServerFinder.Beacon: Attempting to send a beacon." );

                DatagramSocket clientSocket = new DatagramSocket();
                InetAddress IPAddress = InetAddress.getByName("192.168.1.1");

                byte[] sendData = new byte[ 1024 ];
                
                String sentence = "@ABP#";          // App Beacon Protocol
                sendData = sentence.getBytes();
                DatagramPacket sendPacket = new DatagramPacket( sendData, sendData.length, IPAddress, UDP_PORT_APP_TO_SERVER );

                clientSocket.send( sendPacket );

                clientSocket.close();

                System.out.println( "ServerFinder.Beacon: Sent a beacon." );
            } catch ( SocketException e ) {
                System.out.println( "ServerFinder.Beacon: An error occured while opening the socket to send a beacon." );
                System.out.println( e.getMessage() );
            } catch ( UnknownHostException e ) {
                System.out.println( "ServerFinder.Beacon: An error occured while reading own IP Address." );
                System.out.println( e.getMessage() );
            } catch ( IOException e ) {
                System.out.println( "ServerFinder.Beacon: An error occured while sending the beacon." );
                System.out.println( e.getMessage() );
            }
        }
    }

}

// Resources:
//  https://systembash.com/a-simple-java-udp-server-and-udp-client/
//  https://www.geeksforgeeks.org/working-udp-datagramsockets-java/
