// CLASS THREAD NOT ACTIVE IN PRESENTATION / UT VERSION, as server discovery is not needed for stationary demo/does not function in eduroam network (broadcasting blocked)
//
// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "HUB Server"-Application (for Raspberry PI with Java SE Runtime)
//
// "AppFinder"-Class (contains process of main())
// Listens for datagram beacons from smartphone apps wishing to connect to the HubServer and replies.
//  Constructor             :   Opens Datagram Socket (UDP)
//  run method              :   Main Process Method     :    called when UDP socket is online   :   listens to UDP beacons
//  interpretData method    :   processes data from UDP beacons and replies     :   called in Main Process Method run()

// !!! UDP Port: 9970 !!!
//

package com.teamo.biom.hub;

// ----- Imported Libraries -----
    import java.net.*;      // Network Library
    import java.io.*;       // Input/Output Library

public class AppFinder extends Thread
{
    // ----- Common Variables -----
        final int PORT_UDP_APP_TO_SERVER = 9970;
        final int PORT_UDP_SERVER_TO_APP = 9971;

        public DatagramSocket serverSocket;

    // ----- Constructor -----
    public AppFinder()
    {
        try {
            System.out.println( "AppFinder: Attempting to open UDP Socket." );

            // Open UDP Socket
                serverSocket = new DatagramSocket( PORT_UDP_APP_TO_SERVER );

            // Update Controller Status
                Controller.updateUDP( true );

            System.out.println( "AppFinder: The UDP Socket has been opened successfully." );
        } catch (SocketException e) {
            System.out.println( "AppFinder: Could not open UDP Socket." );
            System.out.println( e.getMessage() );
            //Update Controller Status
                Controller.updateUDP( false );
        }
    }

    // ----- Main Process Method -----
    // Listens for incoming UDP beacons.
    // Called in program main.
    public void run()
    {
        while ( true )
        {
            try {
                System.out.println( "AppFinder: Listening for input." );

                byte[] inData = new byte[ 1024 ];                   // Initialize Data Array
                DatagramPacket inPacket =                           // Initialize DatagramPacket for Reception
                    new DatagramPacket( inData, inData.length );

                serverSocket.receive(inPacket);                     // Receive incoming Packets

                String input = new String( inPacket.getData() );    // Convert Packet Data to String

                if ( input != null )
                {
                    interpretData( input, inPacket.getAddress() );    // Pass Data to Interpret Method
                }
                
            } catch ( IOException e ) {
                System.out.println( "AppFinder: Could not receive Datagram properly." );
                System.out.println( e.getMessage() );

                Controller.updateUDP( false );
            } 
        }
    }

    // ----- Data Interpretation Method -----
    // Reads input Strings, checks for relevance, and replies to original sender for connection.
    public void interpretData( String input, InetAddress appIp )
    {
        if ( input.contains("ABP#"))     // Check for incoming data relevance: AppBeaconProtocol
        {
            try {
                System.out.println( "AppFinder: Received connection call from App with the IP " + appIp );

                // Prepare Output Data
                    byte[] outData = new byte[1024];
                    String outputString = "ACP#";           // AppConnectionProtocol
                    outData = outputString.getBytes();

                // Prepare Outgoing Packet
                    DatagramPacket outPacket =
                        new DatagramPacket(outData, outData.length, appIp, PORT_UDP_SERVER_TO_APP);

                // Send Outgoing Packet to original sender.
                    serverSocket.send(outPacket);
                
                System.out.println( "AppFinder: Replied to calling App with the IP " + appIp );
            } catch (IOException e) {
                System.out.println( "AppFinder: Could not reply to calling App with the IP " + appIp );
                System.out.println( e.getMessage() );
            }
        }
    }
}

// Resources:
//  https://systembash.com/a-simple-java-udp-server-and-udp-client/