// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "HUB Server"-Application (for Raspberry PI with Java SE Runtime)
//
// "AppServer" class:
// Establishes the TCP server of the HUB and handles incoming requests from client apps through its "Connector" inner class.
// Responds to requests through the interpretAndAct() method.
//
package com.teamo.biom.hub;

//
// ----- Imported Libraries -----
    import java.net.*;      // Network Library
    import java.io.*;       // Input/Output Library

public class AppServer
{
    // ----- Common Variables -----
        final public int TCP_PORT = 8790;  

        static ServerSocket serverSocket;
    
    // ----- Constructor -----
    // Attempts to establish TCP Server
    // Called in Main Loop whenever Wifi is connected, but the TCP Server is not online.
    AppServer()
    {

        try {
            System.out.println( "AppServer: Attempting to establish TCP Server at " + InetAddress.getLocalHost() );

            if ( serverSocket != null )
                serverSocket.close();

            serverSocket = new ServerSocket( TCP_PORT );   // Attempt opening Socket

            Controller.updateTCP(true);               // Update Controller

            System.out.println( "AppServer: TCP Server successfully established." );
        } catch (IOException e) {
            System.out.println( "AppServer: Error caught when trying to establish ServerSocket on port " + TCP_PORT );
            System.out.println( e.getMessage());

            Controller.updateTCP( false );  // Update Controller
        }
    }

    // Connector inner class
    // Listens for incoming requests from client apps (android app), and relays their input to the interpretAndAct() method
    public class Connector extends Thread
    {
        public void run()
        {
            System.out.println( "AppServer.Connector: Starting to listen for client input." );

            while ( true )
            {
                try (
                    Socket incomingSocket = serverSocket.accept();
                    InputStreamReader in = new InputStreamReader( incomingSocket.getInputStream() );
                    PrintWriter out = new PrintWriter( incomingSocket.getOutputStream(), true );
                ) {

                    interpretAndAct( getMessage( in ), out );

                } catch ( IOException e ) {
                    System.out.println( "AppServer.Connector: An error occured while listening for client activity." );
                    System.out.println( e.getMessage() );
                }
            }
        }
    }

    // Method for interpreting different incoming tcp messages and acting accordingly/responding to the app with the requested data.
    static void interpretAndAct( String[] message, PrintWriter clientReturn )
    {
        if ( message != null )
        {
            String protocol = message[0];
            String content = message[1];
            String[] data = content.split("_");

            switch ( protocol )
            {
                case "DRP":
                    clientReturn.write( HubServer.constructProtocol( "MDP", ModuleList.getModulePackage() ) );  // Module Data Protocole
                    break;
                
                case "UIP":
                    long modId = Long.parseLong( data[0] );
                    int inType = Integer.parseInt( data[1] );
                    int inVal = Integer.parseInt( data[2] );
                    ModuleList.updateActuatorState(modId, inType, inVal);
                    break;

                default:
                    break;
            }
        }
    }

    // Method used to filter incoming tcp streams into relevant protocol messages before relaying them to the interpretAndAct method.
    // Called in connector thread.
    static String[] getMessage( InputStreamReader in )
    {
        try {
            int byteIn = in.read();
            //System.out.println( (char)byteIn );
            if ( byteIn == '@' )
            {
                String protocol = "";
                for ( int i = 0; i < 3; i++)
                    { protocol += (char)in.read(); }

                String content = "";
                byteIn = in.read();
                while ( byteIn != '#' )
                {
                    content += (char)byteIn;
                    byteIn = in.read();
                }
                
                System.out.println( protocol + ": " + content );

                String[] output = { protocol, content };
                return output;
            }
        } catch ( Exception e ) {
            System.out.println( "AppServer: An error occured while producing message from Inputstream." );
            System.out.println( e.getMessage() );
        }
        System.out.println( "Check: Stream unready" );
        return null;
    }
}