// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "HUB Server"-Application (for Raspberry PI with Java SE Runtime)
// [ Current Version only tested on Windows 10 ]
//
// Function:    Primary Algorithmic and Communicative Unit of the 
//              modular greenhouse system. Handles communication to
//              individual greenhouse modules, as well as the user
//              input through a smartphone application. Connects to 
//              wifi-networks through data stored in a config file, 
//              which it can obtain through a serial connection to 
//              a PC-application ("HUB Setup"). Handles behaviour of
//              individual Greenhouse Modules, receives their sensor
//              data, answers, and/or relays to the smartphone app.
//              Constantly running on the HUB-Raspberry Pi.
//
// Program Main Class "HubServer"
//  Starts Threads with specified functions according to Controller Status
//      Main Thread starts and controls the Controller Thread, which handles all sub-threads/services of the HUB server.
//
//  EventListeners:
//      Serial dataReceived     :   listens on COM Port for Serial communication from modules and "HUB Setup"-program

package com.teamo.biom.hub;

// Imports
import java.lang.Thread.State;

public class HubServer
{

    // Main Thread
    public static void main( String[] args )
    {

        // Start Controller Thread
        Controller controller = new Controller();
        controller.start();

        // Variables used for restarting the controller due to performance issues
        int issueCounter = 0;
        Thread terminator = null;

        ModuleList.makeTestBoxes();

        // Temporary solution for quickly finding connected modules ( DOES NOT UPDATE WITH FUTURE DISCONNECTIONS )
        // ONLY IN PRESENTATION / DEMO VERSION ( otherwise controlled by Controller!!! )
        SerialCommunicator.findPort();

        // ----- Main Thread Loop -----
        while ( true ) {
            if ( controller != null )
            {
                if ( controller.getState() == State.TERMINATED )
                {
                    System.out.println( "HubServer: Controller has been terminated. Restarting..." );
                    controller = new Controller();
                    controller.start();
                }
                else if ( controller.getState() != State.RUNNABLE )
                {
                    issueCounter++;
                    System.out.println( "HubServer: Controller is experiencing issues." );
                }

                if ( issueCounter >= 5 )
                {
                    terminator = controller;
                    controller = null;
                    terminator.interrupt();
                    System.out.println( "HubServer: Terminating Controller due to performance issues." );
                }
            } else {
                controller = new Controller();
                controller.start();
            }
        }
    }

    // Basic Method for constructing strings to be sent to the android app.
    static String constructProtocol( String protocol, String[] content )
    {
        String message =        "@";
            message +=          protocol;
        
            for ( int i = 0; i < content.length; i++ )
            {
                message +=      content[i];
                if ( i != content.length-1 )
                    message +=  "_";
            }
            
            message +=          "#";

        return message;
    }

}