// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// Code by Zino J. Vieth
// "HUB Server"-Application (for Raspberry PI with Java SE Runtime)
//
// "SerialCommunicator" class:
// Handles connected modules and their data output.
// Provides a method to transmit single byte messages for actuator control of the modules.


package com.teamo.biom.hub;

// Imported Libraries
    import java.io.InputStreamReader;
    import java.io.PrintWriter;
    import java.util.ArrayList;
    import com.fazecast.jSerialComm.*;

public class SerialCommunicator extends Thread
{
    static long lastCallId;
    static ArrayList<String> usedPorts = new ArrayList<String>();
    static SerialPort[] ports;

    // Method that intitially detects connected modules
    // TEMPORARY NON-THREAD SOLUTION
    // must be updated for new modcall protocol!!!
    static void findPort()
    {
        System.out.println( "SerialCommunicator: Starting up." );

        ports = SerialPort.getCommPorts();

        for ( int i = 0; i < ports.length; i++ )
        {
            if ( !usedPorts.contains( ports[i].toString() ) )
            {
                usedPorts.add( ports[i].toString() );
                new SerialCommunicator( ports[i] ).start();
            }
            
            System.out.println( ports[i].toString() );
        }
    }

    // Method that interprets incoming Sensor data transmissions from the modules and passes their information to the ModuleList class
    public void interpretData( String protocol, String content )
    {
        try {
            switch ( protocol )
            {
                case "SDP":
                    String data[] = content.split("_");
                    long modId = Long.parseLong( data[0] );
                    int sensorId = Integer.parseInt( data[1] );
                    int sensorVal = Integer.parseInt( data[2] );
                    ModuleList.feedSensorData( modId, sensorId, sensorVal );
                    break;

                default:
                    break;
            }
        } catch ( Exception e ) {
            System.out.println( e.getMessage() );
        }
    }

    // -------------------------------------------------------------------------------


    public SerialPort comPort;

    // CONSTRUCTOR / SETUP of serial communicator
    public SerialCommunicator( SerialPort initPort )
    {
        this.comPort = initPort;
        comPort.setBaudRate( 9600 );
        comPort.openPort();
        comPort.setComPortTimeouts(SerialPort.TIMEOUT_READ_SEMI_BLOCKING, 0, 0);
        new Hubcall().start();
    }
    
    // Method that continuously listens for incoming data from connected modules
    public void run()
    {
        try (
            InputStreamReader in = new InputStreamReader( comPort.getInputStream() );
        ) {
            while (true)
            {
                int byteIn = in.read();
                if ( byteIn == '@' )
                {
                    System.out.println( "SerialCommunicator: Reading incoming command." );
                    int count = 0;
                    String protocol = "";
                    String content = "";
                    while ( (byteIn = in.read()) != '#' )
                    {
                        if ( (byteIn > 47 && byteIn < 123 ) )
                        {
                            if ( count < 3 )
                                protocol += (char)byteIn;

                            if ( count >= 3 )
                                content += (char)byteIn;

                            count++;
                        }
                    }
                    System.out.println( protocol + ": " + content );
                    this.interpretData( protocol, content );
                }
                
            }
         } catch (Exception e) { e.printStackTrace(); }
         comPort.closePort();
    }

    // Simple TEMPORARY transmission method
    static void transmit( SerialPort port, String output )
    {
        try (
            PrintWriter out = new PrintWriter( port.getOutputStream(), true );
        ) {
            out.write( output );
            out.flush();
        } catch ( Exception e ) {
            System.out.println( "SerialCommunicator: An error occured when attempting to transmit data." );
            System.out.println( e.getMessage() );
        }
    }

    // Hubcall Thread subclass
    // must be updated to new modcall protocol
    // DISABLE FOR PRESENTATION / DEMO VERSION!!
    public class Hubcall extends Thread
    {
        public void run()
        {
            long timer = 0;
            final long DELAY = 5000;

            try (
                PrintWriter out = new PrintWriter( comPort.getOutputStream(), true );
            ) {
                while ( true )
                {
                    if ( ProcessTimer.globalMS - timer > DELAY)
                    {
                        timer = ProcessTimer.globalMS;
                        SerialCommunicator.lastCallId++;
                        long callId = SerialCommunicator.lastCallId;
                    
                        String[] content = { "" + callId };
                        out.write( HubServer.constructProtocol("HCP", content));
                        out.flush();
                        System.out.println( "SerialCommunicator: Sent a Hubcall.");
                    }
                }
            }
        }
    }

}