// Creative Technology, Module 2: Smart Environments
// Final Project, Team O
// "BI/OM" - Module Code
// To be uploaded with a different id to each module (box).
//
// Code by Willem, Wout, and Zino
// Documented by Zino
//

// Imported Libraries
  // Adafruit Library for digital LED strips
  #include <Adafruit_NeoPixel.h>  

// MODULE SPECIFIC ID and MODULE TYPE, standard is 0, will be changable through app.
// Has to be changed before uploading!!
// Each Module has to have a unique ID.
  const unsigned long MODULE_ID =   10001;
  const unsigned int  MODULE_TYPE = 0;

// SENSOR IDs
  const int SENSOR_ID_HUMIDITY =    1;
  const int SENSOR_ID_WATER_TANK =  2;

// ACTUATOR IDs
  const int ACTUATOR_ID_LED =         1;
  const int ACTUATOR_ID_WATER_PUMP =  2;

// Used Pins
  const int PIN_LED_DIGITAL       = 8;
  const int PIN_LED_ANALOG_RED    = 5;
  const int PIN_LED_ANALOG_GREEN  = 3;
  const int PIN_LED_ANALOG_BLUE   = 6;

  const int PIN_PUMP[4] = {         9,
                                    10,
                                    11,
                                    12
  };

  const int PIN_WATER_TANK_SENSOR[4] = {
                                    A0,
                                    A1,
                                    A2,
                                    A3
  };

  const int PIN_HUMIDITY_SENSOR   = A4;
  

// Global Variables
  int sysMS;  // Variable used to store the current milliseconds since device boot.
              // Used for process timing of sensors and led updates.

  unsigned long sensorTimer;
  const unsigned long SENSOR_TRANSMISSION_DELAY = 2000;


// --------- LED ACTUATOR SECTION ---------

  // Non-Pin Constants
    const int LED_DIGITAL_COUNT = 33;

  // Global Variables
    int ledPower;

    Adafruit_NeoPixel ledStrip = Adafruit_NeoPixel( 
                                  LED_DIGITAL_COUNT,
                                  PIN_LED_DIGITAL, 
                                  NEO_GRB + NEO_KHZ800 );

    unsigned long digLedUpdateTimer;  // Variable used to track the milliseconds between each led strip update


  // Method for switching the led power upon input from hub
  void switchLedPower()
  {
    ledPower++;
    if (ledPower > 1)
      ledPower = 0;
  }

  // Method for outputting LED state
  void actuatorLedOutput()
  {
    // RGB Colour Code for LED colour: purple
      int ledCol[3] = { 255, 0, 255 };

    // Update Analog LED Strip on each tick
      analogWrite( PIN_LED_ANALOG_RED,   ledPower * ledCol[0] );
      analogWrite( PIN_LED_ANALOG_GREEN, ledPower * ledCol[1] );
      analogWrite( PIN_LED_ANALOG_BLUE,  ledPower * ledCol[2] );

    // Update Digital LEDs once every two seconds (avoids flicker)
    unsigned long updateDelay = 2000; // 2000ms = 2s
    if ( sysMS > digLedUpdateTimer + updateDelay )
    {
      int pb;
      for (int i = 0; i < LED_DIGITAL_COUNT; i++) {
        pb = ledCol[2];
        if (pb == 255) pb = 100;

        ledStrip.setPixelColor( i , ledStrip.Color( 
                                      ledPower * ledCol[0],
                                      ledPower * pb,
                                      ledPower * ledCol[1] ) );
        ledStrip.show();  
      }
      
      digLedUpdateTimer = sysMS;
    }
  }

// --------- PUMP ACTUATOR SECTION ---------

  // Constants
    const unsigned long ACTUATOR_PUMP_DURATION = 5000;

  // Global Variables
    bool pumpStarter, pumpRunning;
    unsigned long pumpTimer;

  // Method for updating pump starter upon input from HUB
  void startPump()
  {
    if ( !pumpRunning )
      pumpStarter = true;
  }

  // Method for updating pump status and activity
  // called in main loop
  void updatePumpActuator()
  {
    if ( pumpStarter )
    {
      for ( int i = 0; i < 4; i++ )
        digitalWrite( PIN_PUMP[i], HIGH );

      pumpTimer = sysMS;
      pumpRunning = true;
      pumpStarter = false;
    }

    if ( pumpRunning )
    {
      if ( sysMS >= pumpTimer + ACTUATOR_PUMP_DURATION)
      {
        pumpRunning = false;

        for ( int i = 0; i < 4; i++ )
          digitalWrite( PIN_PUMP[i], LOW );
      }
    }
  }

// HUMIDITY SENSOR SECTION

  int getHumidity()
  {
    int humidity = analogRead( PIN_HUMIDITY_SENSOR );
      humidity = max( 300, humidity );
      humidity = min( 700, humidity );

    return humidity;
  }

// --------- WATER TANK SENSOR SECTION ---------

  int getWaterTankLevel()
  {
    int tankLevel = 0;

    for ( int i = 0; i < 4; i++ )
    {
      if ( analogRead( PIN_WATER_TANK_SENSOR[i] ) == 0 )
        tankLevel = i;
    }
    
    return tankLevel;
  }


// --------- SERIAL COMMUNICATION SECTION ---------
// Methods and Variables related to transmitting and receiving data/commands through serial.
// -> Everything related to communicating with the HUB.

  // Method that creates a string for Serial transmission to the HUB.
  // String format is specific for the hub to recognize and interpret according to protocol tag.
  // Resulting String: "@(Protocol Tag)(data0)_(data1)_(data2)#" (with variable amount of data)
    String constructProtocol( String protocol, String *content, int contentCount )
    {
      String msg = "@" + protocol;
      
      for ( int i = 0; i < contentCount; i++ )
      {
        msg +=  content[i];
        msg += "_";
      }

      msg = msg.substring( 0, msg.length()-1 );
      msg +=  "#";

      return msg;
    } 

  // Method for reading and interpreting incoming serial data.
  // called in main loop
  void serialListenAndActuate()
  {
    while ( Serial.available() > 0 )
    {
      int byteIn = Serial.read();

      switch ( byteIn )
      {
        case 49: // Input = '1'
          startPump();
          break;

        case 50: // Input = '2'
          switchLedPower();
          break;
      }
    }
  }


// --------- PROGRAM SETUP ---------
void setup()
{

  // LED ACTUATOR SETUP
    // Adafruit Digital Strip
      ledStrip.begin();
    // Pins
      pinMode( PIN_LED_ANALOG_RED,    OUTPUT );
      pinMode( PIN_LED_ANALOG_GREEN,  OUTPUT );
      pinMode( PIN_LED_ANALOG_BLUE,   OUTPUT );

  // PUMP ACTUATOR SETUP
    // Pins  
      for ( int i = 0; i < 4; i++ )
        pinMode( PIN_PUMP[i], OUTPUT );

  // HUMIDITY SENSOR SETUP
    // Pins
      pinMode( PIN_HUMIDITY_SENSOR, INPUT );

  // WATER TANK SENSOR SETUP
    // Pins
      for ( int i = 0; i < 4; i++ )
        pinMode( PIN_WATER_TANK_SENSOR[i], INPUT );

  // Serial Setup
    Serial.begin(9600);

  // Send Mod Call Protocol to HUB
  // Allows HUB to initially register this module as part of the system
  // and recognize future transmissions from this module.
    String content[2] = {
      String(MODULE_ID),
      String(MODULE_TYPE)
    };
    Serial.print( constructProtocol( "MCP", content, 2 ) );  
}

// --------- MAIN LOOP ---------
void loop() {
  // Updated Millisecond Storage
    sysMS = millis();

  // Transfer Sensor Data to Hub every two seconds
    if ( sysMS > sensorTimer + SENSOR_TRANSMISSION_DELAY ) {

      // Prepare Protocol Content Arrays
      String sdpContent[3] = { 
        String(MODULE_ID),
        "",
        ""
      };

      // Update Protocol Content Array with Humidity reading
      sdpContent[1] = String( SENSOR_ID_HUMIDITY );
      sdpContent[2] = String( getHumidity() );
      Serial.print( constructProtocol( "SDP", sdpContent, 3 ) ); // Transmit Protocol

      // Update Protocol Content Array with Water Tank reading
      sdpContent[1] = String( SENSOR_ID_WATER_TANK );
      sdpContent[2] = String( getWaterTankLevel() );
      Serial.print( constructProtocol( "SDP", sdpContent, 3 ) ); // Transmit Protocol
      
      // Reset Sensor Transmission Timer
      sensorTimer = sysMS;
    }
  
  // Actuator Update Methods
    actuatorLedOutput();
    updatePumpActuator();

  // Serial Input Reading Method
    serialListenAndActuate();
}